;*************************************************;
; Ping Pong! by Leigh Prosho                      ;
; v1.0 Started 21/9/01                            ;
; v1.0 Finished 22/9/01                           ;
; v1.0 - basic 2 player pong game                 ;
; v1.1 Started 23/9/01                            ;
; v1.1 Finished 23/9/01                           ;
; v1.1 - include single player mode               ;
; v1.2 Started 25/9/01                            ;
; v1.2 Finished 27/9/01                           ;
; v1.2 - recoded completely, revised gfx slightly ;
;*************************************************;

;*******************************;
; set up the constant variables ;
;*******************************;

; General constants

Const width=640,height=480								; holds the screen resolution

; Keyboard constants

Const p1up=30, p1down=44								; holds A & Z
Const p2up=37, p2down=50								; holds K & M
Const space=57, esc=1									; holds Spacebar & Esc
Const ret=28, I=23										; holds return & I

;*******************************;
; set up global game  variables ;
;*******************************;

; Global game variables

Global player1score=0									; holds player 1 score 7 sets value to zero
Global player2score=0									; holds player 2 score and sets value to zero
Global players=0

; Global sound variables & loads

Global beep1											; holds the wall bounce sound
Global beep2											; holds the bat bounce sound

; Global gfx variables & loads

Global background										; holds the background gfx
Global wall												; holds the wall gfx
Global bat												; holds the bat gfx
Global ball												; holds the ball gfx
Global instructions										; holds the instruction screen
Global logo												; holds the logo gfx
Global play												; holds the "space to play" gfx


;**************;
; set up types ;
;**************;

;build the BAT type

Type Bat
	Field batx, baty									; holds the x,y screen coordinates of the bat
End Type

; build the BALL type

Type Ball
	Field ballx, bally									; holds the x,y screen coordinates of the ball
	Field ballxdir, ballydir							; holds the x,y variables for the ball direction
End Type

; set up new elements of BAT type for each player, thats global

Global Player1.Bat=New Bat								; creates a new element type for player 1's bat
Global Player2.Bat=New Bat								; creates a new element type for player 2's bat
Global Newball.Ball=New Ball

; Set screen size

Graphics width,height									; this sets the screen resolution, using variables width & height

; Draw to back buffer

SetBuffer BackBuffer()									

; Load all graphics files

loadgfx()												; this runs the function that loads the game graphics into memory

; Load all sound files

loadsfx()												; this runs the function that loads the game sounds into memory

; initialise all game variables

setvariables()											; this runs the function that loads the game variables

; run the intro screen

intro ()												; this runs the function that displays the instruction screen


;################;
; Main Game Loop ;
;################;


startgame()												; this runs the function that sets up the start of the game

maingame()												; this runs the function that forms the main game loop

endgame()												; this runs the function that finishes the game



;************************************;
; all game functions are stored here ;
;************************************;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function loads all the graphics into memory ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function loadgfx()										

	background=LoadImage("gfx\background.bmp")			; this loads the background image into memory
	wall=LoadImage("gfx\wall.bmp")						; this loads the wall image into memory
	bat=LoadImage("gfx\bat.bmp")						; this loads the bat image into memory
	ball=LoadImage("gfx\ball.bmp")						; this loads the ball image into memory
	instructions=LoadImage("gfx\instructions.bmp")		; this loads the instructions screen into memory
	logo=LoadImage("gfx\logo.bmp")						; this loads the logo image into memory
	play=LoadImage("gfx\s2play.bmp")					; this loads the "space to play" image into memory
	SetFont LoadFont("tahoma",16,True )					; this sets and loads the font, size 18 points, bold

End Function 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function loads all the sound files into memory ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function loadsfx()

	beep1=LoadSound("sfx/beep.wav")						; this loads the wall bounce sound into memory
	beep2=LoadSound("sfx/beeplow.wav")					; this loads the bat bounce sound into memory

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function sets the game variables for bat & ball position etc ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function setvariables()

	Player1\batx=8										; this sets the X screen coordinate of Player 1's bat
	Player1\baty=(height/2)								; this sets the Y screen coordinate of Player 1's bat
	Player2\batx=624									; this sets the X screen coordinate of Player 2's bat
	Player2\baty=(height/2)								; this sets the Y screen coordinate of Player 2's bat
	Newball\ballx=width/2								; this sets the X screen coordinate of the ball
	Newball\bally=height/2								; this sets the Y screen coordinate of the ball
	Newball\ballxdir=Rnd(1,4)							; this sets the horizontal direction of the ball
	Newball\ballydir=Rnd(1,4)							; this sets the vertical direction of the ball
	If Newball\ballydir=3 Then Newball\ballydir=-1		; this check the random number generated, and assigns a new value
	If Newball\ballydir=4 Then Newball\ballydir=-2		; this check the random number generated, and assigns a new value
	If Newball\ballxdir=2 Then Newball\ballxdir=-1		; this check the random number generated, and assigns a new value
	If Newball\ballxdir=4 Then Newball\ballxdir=-2		; this check the random number generated, and assigns a new value

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function displays the scores on screen ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function showscore()
	
	Text(24,0,"Player 1 Score = "+player1score)			; display text & player 1 score on screen at x,y screen coordinates
	Text(475,0,"Player 2 Score = "+player2score)		; display text & player 2 score on screen at x,y screen coordinates

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function draws the gfx to the backbuffer ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function displaygfx()

	MidHandle bat										; sets the image handle to the centre of the image
	MidHandle ball										; sets the image handle to the centre of the image
	DrawImage(background,0,0)							; draws the background gfx to the backbuffer
    DrawImage(wall,0,32)								; draws the wall gfx to the backbuffer at x,y screen coordinates
    DrawImage(wall,0,480-32)							; draws the wall gfx to the backbuffer at x,y screen coordinates
	DrawImage(bat,Player1\batx,Player1\baty)			; draws the bat gfx to the backbuffer at x,y screen coordinates
    DrawImage(bat,Player2\batx,Player2\baty)			; draws the bat gfx to the backbuffer at x,y screen coordinates
    DrawImage(ball,Newball\ballx,Newball\bally)			; draws the ball gfx to the backbuffer at x,y screen coordinates

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function checks to see if bat movement keys have been pressed, and updates variables ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function batmove()

	Player1\baty=Player1\baty+(KeyDown(p1down)-KeyDown(p1up))*3														; checks if keys A or Z pressed, and increases / decreases Player 1 bat Y coord accordingly
 	If players=1 And Newball\bally>Player2\baty And Newball\ballxdir>=1 Then Player2\baty=Player2\baty+1*Rnd(2,3)	; if number of players is 1, checks if ball Y coord is greater than Player 2 bat Y coord AND if the ball is moving towards Player 2 bat, then increases Player 2 bat Y coord accordingly
	If players=1 And Newball\bally<Player2\baty And Newball\ballxdir>=1 Then Player2\baty=Player2\baty-1*Rnd(2,3)   ; if number of players is 1, checks if ball Y coord is greater than Player 2 bat Y coord AND if the ball is moving towards Player 2 bat, then increases Player 2 bat Y coord accordingly
	If players=2 Then Player2\baty=Player2\baty+(KeyDown(p2down)-KeyDown(p2up))*3									; if number of players is 2, checks if keys K or M pressed, and increases / decreases Player 2 bat Y coord accordingly

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function updates the ball position ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function ballmove()

	Newball\ballx=Newball\ballx+Newball\ballxdir		; updates ball X coordinate
    Newball\bally=Newball\bally+Newball\ballydir		; updates ball Y coordinate

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function performs the collision detection for the bats and ball, ;
; and updates the bats & ball X and Y coordinates as needed             ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function collisiondetect()

	If ImagesCollide(bat,Player1\batx,Player1\baty,0,wall,0,32,0)							; checks if Player 1's bat hits the upper wall
		Player1\baty=80																		; if so, set Y coordinate for Player 1 bat to 80
	EndIf
	
	If ImagesCollide(bat,Player1\batx,Player1\baty,0,wall,0,height-32,0)					; checks if Player 1's bat hits lower wall
		Player1\baty=height-64																; if so, set Y coordinate for Player 1 bat to 352
	EndIf
		
	If ImagesCollide(bat,Player2\batx,Player2\baty,0,wall,0,32,0)							; checks if Player 2's bat hits the upper wall
		Player2\baty=80																		; if so, set Y coordinate for Player 2 bat to 80
	EndIf
	
	If ImagesCollide(bat,Player2\batx,Player2\baty,0,wall,0,height-32,0)					; checks if Player 2's bat hits lower wall
		Player2\baty=height-64																; if so, set Y coordinate for Player 1 bat to 352
	EndIf
	
	If ImagesCollide(ball,Newball\ballx,Newball\bally,0,bat,Player1\batx,Player1\baty,0)	; checks if ball hits Player 1's bat
		Newball\ballxdir=Rnd(1,4)															; if so, alter ball horizontal direction
		PlaySound beep2																		; play sound
	EndIf
	
	If ImagesCollide(ball,Newball\ballx,Newball\bally,0,bat,player2\batx,Player2\baty,0)	; check is ball hits Player 2's bat
		Newball\ballxdir=Rnd(1,4)*-1														; if so, alter ball horizontal direction
		PlaySound beep2																		; play sound
	EndIf
	
	If ImagesCollide(ball,Newball\ballx,Newball\bally,0,wall,0,32,0)						; checks if ball his upper wall
		Newball\ballydir=Rnd(1,4)															; if so, alter ball vertical direction
		PlaySound beep1																		; play sound
	EndIf
	
	If ImagesCollide(ball,Newball\ballx,Newball\bally,0,wall,0,480-32,0)					; checks if ball hits lower wall
		Newball\ballydir=Rnd(1,4)*-1														; if so, alter ball vertical direction
		PlaySound beep1 																	; play sound
	EndIf
	
End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function checks to see if the ball gets past the bats, and if so increases the player score ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function checkifscored()

	If Newball\ballx>=640										; checks if ball X coordinate is greather than or equal to 640
	   player1score=player1score+1								; if so, increases Player 1 score by 1 point
	   Text((width/3)+25,16,"Player 1 scores a point!")			; if so, displays a text message
	   Flip														; flip backbuffer to front, displaying gfx drawn
	   Delay 1500												; pause game for a second or so
	   startgame()												; this runs the function that sets up the start of the game
	EndIf
	
	If Newball\ballx<=0											; checks if ball X coordinate is less than or equal to 0								
	   player2score=player2score+1								; if so, increases Player 2 score by 1 point
	   Text((width/3)+25,16,"Player 2 scores a point!")			; if so, displays a text message
	   Flip														; flip backbuffer to front, displaying gfx drawn
	   Delay 1500												; pause game for a second or so
	   startgame()												; this runs the function that sets up the start of the game
	EndIf
	
End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function displays the instruction screen, and waits until the SPACE bar is pressed before continuing ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function intro()

 While players=0																			

	Cls																						; clears the screen
	MidHandle logo																			; sets the image handle to the centre of the image
  	DrawImage(background,0,0)																; draws the background image to the backbuffer
	DrawImage(logo,width/2,50)																; draws the logo image to the backbuffer
	Text(12,height/4,"How many players? (1 or 2)")											; displays a text message
    If KeyDown(2) Then players=1															; checks if the 1 key is pressed, if so sets players variable to 1
    If KeyDown(3) Then players=2															; checks if the 2 key is pressed, if so sets players variable to 2
  	Flip																					; flip backbuffer to front, displaying gfx drawn
    If KeyDown(esc) Then End																; checks if the ESC key is pressed, if so quit to windows

 Wend																						; loop only while players variable = 0

 While Not KeyDown(space)

  	Text(12,height/3,"Number of players selected ="+players)								; displays a text message
  	Text(12,height/2.5,"Please press the SPACE key to play, or the I key for instructions")	; displays a text message
  	Flip																					; flip backbuffer to front, displaying message drawn
  	If KeyDown(I) Then instructions()														; checks if the I key is pressed, if so runs the instruction function
    If KeyDown(esc) Then End																; check if the ESCAPE key is pressed, if so quit to windows

 Wend																						; loop until SPACE key pressed

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function displays the starting screen / player select screen, and waits until the number of ; 
; players is selected                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function instructions()

Repeat

	Cls													; clears the screen
	MidHandle instructions								; sets image handle to the centre of the image
	MidHandle logo										; sets image handle to the centre of the image
	MidHandle play										; sets image handle to the centre of the image
	DrawImage(background,0,0)							; draws the background gfx to the backbuffer
	DrawImage(logo,width/2,50)							; draws the logo image to the backbuffer
	DrawImage(instructions,width/2,height/2)			; draws the indtructions image to the backbuffer
	DrawImage(play,width/2,430)							; draws the play image to the backbuffer
	Flip												; flip backbuffer to front, displaying gfx drawn

Until KeyDown(space)									; loop until SPACE key pressed

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this funtion sets up the start of the game, drawing the screen, the bats, balls, scores etc ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function startgame()

 While Not KeyDown(esc)

	Cls													; clears the screen
	setvariables()										; this runs the function that resets the x & y coordinate variables for the bat & ball to the defaults
	displaygfx()										; this runs the function that draws the game gfx to the backbuffer
	showscore()											; this runs the function that displays the player scores
	If player1score=10 Or player2score=10 Then endgame(); check to see if eithe player score is 10, if so run endgame function				
	Flip												; flip backbuffer to front, displaying gfx drawn
	Delay 2500 : maingame()								; delay, then run maingame function

 Wend													; loop until ESC key pressed

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function creates the main game loop ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function maingame()

 While Not KeyDown(esc)

	Cls													; clears the screen
	displaygfx()										; this runs the function that draws the game gfx to the backbuffer							
	showscore()											; this runs the function that displays the player scores
	batmove()											; this runs the function that checks to see if players have pressed a key to move tha bats
	ballmove()											; this runs the function that moves the ball
	collisiondetect()									; this runs the function that checks to see if the bats / balls / walls collide
	checkifscored()										; this runs the function that checks to see if a either player scores
	Flip												; flip backbuffer to front, displaying gfx drawn
	
 Wend													; loop until ESCAPE key pressed	

 End													; quit to windows

End Function


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; this function ends the game ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function endgame()

 While Not KeyDown(esc)

	Cls													; clears the screen
	displaygfx()										; this runs the function that draws the game gfx to the backbuffer
    showscore()											; this runs the function that displays the player scores

	If player1score=10									; checks if Player 1 score equals 10			
	   Text((width/3)+40,16,"Player 1 Wins!")           ; if it does, display a text message
	EndIf
	
	If player2score=10									; checks to see if Player 2 score equals 10
	   Text((width/3)+40,16,"Player 2 Wins!")			; if it does, display a text message
	EndIf

	Text(width/4, height/2,"Would you like another game? (Y / N)")	; displays a text message
	Flip												; flip backbuffer to front, displaying gfx drawn
	
	If KeyDown(21)										; checks if the Y key os pressed
	   player1score=0									; if so, reset Player 1 score to zero
	   player2score=0									; if so, reset player 2 score to zero	
	   startgame()										; this runs the function that sets up the start of the game
	EndIf
	
	If KeyDown(49)										; checks if the N key is pressed
	   End												; if so, quits to windows
	EndIf

 Wend													; loop until ESCAPE key pressed

End														; quit to windows

End Function